import os

from boom.api import API
from boom.models import Message


class ConversationService(API):

    def __init__(self):
        self.url = os.getenv("BOOM_CONVERSATION_URL", "https://conversation.boom.app")

    def receive(self, message):
        response = ConversationResponse()
        api_response = self.post("message", message)[0]
        response.message = Message(**api_response["message"])

        for item in api_response["context"]:
            response.context.append(Message(**item))

        for item in api_response["responses"]:
            response.responses.append(Message(**item))

        return response


class ConversationResponse(object):

    def __init__(self):
        self.message = None
        self.context = []
        self.responses = []



from declaration import fields, models


class Code(models.DeclarativeBase):
    scope = fields.StringField()
    code = fields.StringField()


class Account(models.DeclarativeBase):
    id = fields.UUIDField()
    email = fields.StringField()
    first_name = fields.StringField()
    last_name = fields.StringField()
    code = fields.NestedField(Code)
    created_date = fields.DateTimeField()
    updated_date = fields.DateTimeField()


class Platform(models.DeclarativeBase):
    id = fields.UUIDField()
    identifier = fields.StringField()


class Conversation(models.DeclarativeBase):
    id = fields.UUIDField()
    platform = fields.NestedField(Platform)
    account = fields.NestedField(Account)
    created_date = fields.DateTimeField()
    updated_date = fields.DateTimeField()


class Message(models.DeclarativeBase):
    conversation = fields.NestedField(Conversation)
    platform = fields.NestedField(Platform)
    sender = fields.StringField()
    receiver = fields.StringField()
    identifier = fields.StringField()
    intent = fields.StringField()
    content = fields.StringField()
    raw = fields.StringField()
    extra = fields.JSONField()
    timestamp = fields.DateTimeField()



import json
import os

import requests
import six

from boom import encoder


class API(object):
    urls = {
        "index": "/",
        "account": "/account/{client}",
        "authenticate": "/account/authenticate",
        "client": "/client/{client}",
        "client-location": "/client/{client}/location/{location}",
        "client-location-experience": "/client/{client}/location/{location}/experience/{experience}",
        "client-location-platform": "/client/{client}/location/{location}/platform/{platform}",
        "conversation": "/conversation/{conversation}",
        "conversation-link": "/conversation/{conversation}/link/{code}",
        "experience": "/experience/{experience}",
        "message": "/message/{message}",
        "payment": "/order/{order}/payment/{payment}",
        "payment-credentials": "/client/{client}/location/{location}/credentials",
        "payment-method": "/payment/method/{payment_method}",
        "platform": "/platform/{platform}",
        "order": "/order/{order}",
        "order-payment": "/order/{order}/payment/{payment}",
        "order-payment-method": "/order/{order}/payment/method/{method}",
        "order-search": "/order/search",
        "order-status-update": "/order/status/{status}",
        "receipt-send": "/order/{order}/payment/{payment}/receipt",
        "refund": "/order/{order}/payment/{payment}/refund/{refund}",
        "wallet": "/payment/wallet/{identifier}/consume",
        "wallet-adjust": "/payment/wallet/{identifier}/adjust"
    }

    def __init__(self, authorization=None):
        self.api_url = self.url = os.getenv("BOOM_API_URL", "https://api.boom.app")
        self.authorization = authorization

    def get_headers(self):
        headers = {
            "Content-Type": "application/json"
        }

        if self.authorization:
            headers["Authorization"] = self.authorization

        return headers

    def make_url(self, path, querystring=None):
        if path.startswith("/"):
            path = path[1:]

        if querystring:
            querystring = "?{0}".format(six.moves.urllib.parse.urlencode(querystring))
        else:
            querystring = ""

        return "{0}/{1}{2}".format(self.url, path, querystring)

    def generate_endpoint_url(self, endpoint, data=None, **kwargs):
        # Normalize the URL
        path = self.urls.get(endpoint, "")
        if path.startswith("/"):
            path = path[1:]

        querystring = {}
        data = data or {}

        # Replace all tokens in the API URL with the values from data
        for key, value in six.iteritems(kwargs):
            path, updated = self.update_path(path, key, value)
            if not updated and value:
                querystring[key] = value

        for key, value in six.iteritems(data):
            path, updated = self.update_path(path, key, value)

        # If the path ends with a token, that means we are probably creating
        # and don't have a specific item we are working with. Remove the trailing token.
        if path.endswith("}"):
            path = path[0:path.rfind("/")]

        url = self.make_url(path, querystring)
        return url

    @staticmethod
    def update_path(path, key, value):
        # Wrap the key in {}, like client => {client}
        updated = False
        if key.endswith("_id"):
            key = key[:-3]
        token = "{" + key + "}"
        if token in path:
            updated = True
            path = path.replace(token, value)

        return path, updated

    @staticmethod
    def normalize_data(data):
        if isinstance(data, dict):
            data = [data]

        return data

    @staticmethod
    def encode_data(data):
        data = json.dumps(data, cls=encoder.BoomJSONEncoder)
        return data

    def get(self, endpoint, **kwargs):
        url = self.generate_endpoint_url(endpoint, **kwargs)
        headers = self.get_headers()
        dataset = requests.get(url, headers=headers)
        dataset = dataset.json()
        dataset = self.normalize_data(dataset)

        return dataset

    def post(self, endpoint, data, **kwargs):
        headers = self.get_headers()
        data = self.encode_data(data)
        url = self.generate_endpoint_url(endpoint, **kwargs)
        dataset = requests.post(url, headers=headers, data=data)
        dataset = dataset.json()

        return dataset

    def patch(self, endpoint, data, **kwargs):
        headers = self.get_headers()
        data = self.encode_data(data)
        url = self.generate_endpoint_url(endpoint, **kwargs)
        dataset = requests.patch(url, headers=headers, data=data)
        dataset = dataset.json()

        return dataset

    def delete(self, endpoint, data, **kwargs):
        headers = self.get_headers()
        data = self.encode_data(data)
        url = self.generate_endpoint_url(endpoint, **kwargs)
        requests.delete(url, headers=headers, data=data)

        return True



import os

from boom.api import API


class OrderService(API):

    def __init__(self):
        self.url = os.getenv("BOOM_ORDER_URL", "https://order.boom.app")


class OrderResponse(object):

    def __init__(self):
        self.order = None



import decimal
import json


class BoomJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, decimal.Decimal):
            # Note: this is OK for Python 2.7+ due to better precision in float conversion and rounding
            return float(obj)

        return json.JSONEncoder.default(self, obj)



from setuptools import setup, find_packages


classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Topic :: Software Development :: Libraries',
]

setup(
    name='boom-sdk',
    author='Jordan Ambra',
    author_email='jordan@serenitysoftware.io',
    url='https://github.com/boomletsgo/boom-sdk-python',
    version='0.4.2',
    classifiers=classifiers,
    description='SDK for interacting with Boom',
    keywords='boom',
    packages=["boom"],
    install_requires=["six>=1.0.0", "declaration>=0.2.5"],
    include_package_data=True,
    license='The Unlicense',
)



import os

from boom.api import API


class PayService(API):

    def __init__(self):
        self.url = os.getenv("BOOM_PAY_URL", "https://pay.boom.app")


class PayResponse(object):

    def __init__(self):
        self.pay = None
